@@ -32,16 +32,8 @@ use Glib::Object::Subclass Gtk3::Dialog::,
     ),
   ];
 
-our $VERSION = '2.5.1';
-my $EMPTY    = q{};
-my $HEXREGEX = qr{^(.*)           # start of message
-                  \b0x[[:xdigit:]]+\b # hex (e.g. address)
-                  (.*)$           # rest of message
-                 }xsm;
-my $INTREGEX = qr{^(.*)           # start of message
-                  \b[[:digit:]]+\b # integer
-                  (.*)$           # rest of message
-                 }xsm;
+our $VERSION = '2.5.4';
+my $EMPTY = q{};
 
 sub INIT_INSTANCE {
     my $self = shift;
@@ -124,22 +116,6 @@ sub dump_or_stringify {
     );
 }
 
-# External tools sometimes throws warning messages including a number,
-# e.g. hex address. As the number is very rarely the same, although the message
-# itself is, filter out the number from the message
-
-sub filter_message {
-    my ($message) = @_;
-    $message =~ s/\s+$//xsm;
-    while ( $message =~ /$HEXREGEX/xsmo ) {
-        $message =~ s/$HEXREGEX/$1%%x$2/xsmo;
-    }
-    while ( $message =~ /$INTREGEX/xsmo ) {
-        $message =~ s/$INTREGEX/$1%%d$2/xsmo;
-    }
-    return $message;
-}
-
 1;
 
 __END__
