@@ -11,11 +11,12 @@ use Gscan2pdf::EntryCompletion;
 use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use Date::Calc qw(Today Today_and_Now);
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
+use Encode;
 use Readonly;
 Readonly my $ENTRY_WIDTH_DATE     => 10;
 Readonly my $ENTRY_WIDTH_DATETIME => 19;
 
-our $VERSION = '2.5.1';
+our $VERSION = '2.5.4';
 my $EMPTY           = q{};
 my $DATE_FORMAT     = '%04d-%02d-%02d';
 my $DATETIME_FORMAT = '%04d-%02d-%02d %02d:%02d:%02d';
@@ -166,6 +167,12 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, properties => [
         'auto',                                              # default
         [qw/readable writable/]                              # flags
     ),
+    Glib::ParamSpec->scalar(
+        'available-fonts',                                   # name
+        'Available fonts',                                   # nick
+        'Hash of true type fonts available',                 # blurb
+        [qw/readable writable/]                              # flags
+    ),
     Glib::ParamSpec->string(
         'pdf-font',                                           # name
         'PDF font',                                           # nick
@@ -187,13 +194,6 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, properties => [
         undef,                                                # default
         [qw/readable writable/]                               # flags
     ),
-    Glib::ParamSpec->string(
-        'pdf-owner-password',                                 # name
-        'PDF owner password',                                 # nick
-        'PDF owner password',                                 # blurb
-        undef,                                                # default
-        [qw/readable writable/]                               # flags
-    ),
 ];
 
 sub SET_PROPERTY {
@@ -496,7 +496,7 @@ sub add_image_type {
     # Image type ComboBox
     my $hboxi = Gtk3::HBox->new;
     $vbox->pack_start( $hboxi, FALSE, FALSE, 0 );
-    my $label = Gtk3::Label->new( __('Image type') );
+    my $label = Gtk3::Label->new( __('Document type') );
     $hboxi->pack_start( $label, FALSE, FALSE, 0 );
 
     my @image_types = (
@@ -797,54 +797,82 @@ sub add_pdf_options {
     );
     $hbox->pack_end( $combob, FALSE, FALSE, 0 );
 
-    # Font for non-ASCII text
-    my $scwinf = Gtk3::ScrolledWindow->new;
-    $scwinf->set_policy( 'automatic', 'automatic' );
-    $vboxp->pack_start( $scwinf, TRUE, TRUE, 0 );
+    # It would be nice to use a Gtk3::FontButton here, but as we can only use
+    # TTF, and we have to know the filename of the font, we must filter the
+    # list of fonts, and so we must use a Gtk3::FontChooserDialog
     my $hboxf = Gtk3::HBox->new;
-    $scwinf->add_with_viewport($hboxf);
-    $scwinf->get_child->set_shadow_type('none');
+    $vboxp->pack_start( $hboxf, TRUE, TRUE, 0 );
     $label = Gtk3::Label->new( __('Font for non-ASCII text') );
     $hboxf->pack_start( $label, FALSE, FALSE, 0 );
-    my @fonts;
-    my ( undef, $stdout ) =
-      Gscan2pdf::Document::exec_command( ['fc-list : family style file'] );
+    my $fontb = Gtk3::Button->new('Font name goes here');
+    $hboxf->pack_end( $fontb, FALSE, TRUE, 0 );
 
-    my $font = $self->get('pdf-font');
-    for ( split /\n/sm, $stdout ) {
-        if (/ttf:[ ]/xsm) {
-            my ( $file, $family, $style ) = split /:/xsm;
-            chomp $style;
-            $family =~ s/^[ ]//xsm;
-            $style =~ s/^style=//xsm;
-            $style =~ s/,.*$//xsm;
-            my $family_style = "$family $style";
-            push @fonts, [ $file, $family_style, $family_style ];
-            if ( not defined $font
-                and $family_style eq 'Times New Roman Regular' )
-            {
-                $font = $file;
-            }
-        }
+    my $ttffile = $self->get('pdf-font');
+    my $fonts   = $self->get('available-fonts');
+    if (    defined $ttffile
+        and defined $fonts
+        and defined $fonts->{by_file}{$ttffile} )
+    {
+        my ( $family, $style ) = @{ $fonts->{by_file}{$ttffile} };
+        $fontb->set_label("$family $style");
     }
-    @fonts = sort { $a->[1] cmp $b->[1] } @fonts;
-    my $combof = Gscan2pdf::ComboBoxText->new_from_array(@fonts);
-    $combof->signal_connect(
-        changed => sub {
-            $self->set( 'pdf-font', $combof->get_active_index );
+    else {
+        $fontb->set_label( __('Core') );
+    }
+    $fontb->signal_connect(
+        clicked => sub {
+            my $fontwin =
+              Gtk3::FontChooserDialog->new( 'transient-for' => $self, );
+            $fontwin->set_filter_func(
+                sub {
+                    my ( $family, $face ) = @_;
+                    $family = $family->get_name;
+                    $face   = $face->get_face_name;
+                    if (    defined $fonts->{by_family}{$family}
+                        and defined $fonts->{by_family}{$family}{$face} )
+                    {
+                        return TRUE;
+                    }
+                    return;
+                }
+            );
+            if (    defined $ttffile
+                and defined $fonts
+                and defined $fonts->{by_file}{$ttffile} )
+            {
+                my ( $family, $style ) = @{ $fonts->{by_file}{$ttffile} };
+                my $font = $family;
+                if ( defined $style and $style ne $EMPTY ) {
+                    $font .= " $style";
+                }
+                $fontwin->set_font($font);
+            }
+            $fontwin->show_all;
+            if ( $fontwin->run eq 'ok' ) {
+                my $family = $fontwin->get_font_family->get_name;
+                my $face   = $fontwin->get_font_face->get_face_name;
+                if (    defined $fonts->{by_family}{$family}
+                    and defined $fonts->{by_family}{$family}{$face} )
+                {
+
+                    # also set local variable as a sort of cache
+                    $ttffile = $fonts->{by_family}{$family}{$face};
+                    $self->set( 'pdf-font', $ttffile );
+                    $fontb->set_label("$family $face");
+                }
+            }
+            $fontwin->destroy;
         }
     );
-    $combof->set_active_index($font);
-    $hboxf->pack_start( $combof, FALSE, FALSE, 0 );
 
     if ( $self->get('can-encrypt-pdf') ) {
-        my $passb = Gtk3::Button->new( __('Set passwords') );
+        my $passb = Gtk3::Button->new( __('Encrypt PDF') );
         $vboxp->pack_start( $passb, TRUE, TRUE, 0 );
         $passb->signal_connect(
             clicked => sub {
                 my $passwin = Gscan2pdf::Dialog->new(
                     'transient-for' => $self,
-                    title           => __('Set passwords'),
+                    title           => __('Set password'),
                 );
                 $passwin->set_modal(TRUE);
                 my $passvbox = $passwin->get_content_area;
@@ -861,22 +889,10 @@ sub add_pdf_options {
                     $userentry->set_text( $self->get('pdf-user-password') );
                 }
                 $grid->attach( $userentry, 1, $row++, 1, 1 );
-
-                $hbox  = Gtk3::HBox->new;
-                $label = Gtk3::Label->new( __('Owner password') );
-                $hbox->pack_start( $label, FALSE, FALSE, 0 );
-                $grid->attach( $hbox, 0, $row, 1, 1 );
-                my $ownerentry = Gtk3::Entry->new;
-                if ( defined $self->get('pdf-owner-password') ) {
-                    $ownerentry->set_text( $self->get('pdf-owner-password') );
-                }
-                $grid->attach( $ownerentry, 1, $row++, 1, 1 );
                 $passwin->add_actions(
                     'gtk-ok',
                     sub {
                         $self->set( 'pdf-user-password', $userentry->get_text );
-                        $self->set( 'pdf-owner-password',
-                            $ownerentry->get_text );
                         $passwin->destroy;
                     },
                     'gtk-cancel',
