@@ -9,6 +9,7 @@ use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
 use Storable qw(dclone);
 use feature 'switch';
+use GooCanvas2;
 use Gscan2pdf::ComboBoxText;
 use Gscan2pdf::Dialog;
 use Gscan2pdf::Scanner::Options;
@@ -269,7 +270,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.5.1';
+our $VERSION = '2.5.4';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -291,6 +292,7 @@ sub INIT_INSTANCE {
 
     # Notebook to collate options
     $self->{notebook} = Gtk3::Notebook->new;
+    $self->{notebook}->set_scrollable(TRUE);
     $vbox->pack_start( $self->{notebook}, TRUE, TRUE, 0 );
 
     # Notebook page 1
@@ -851,8 +853,6 @@ sub SET_PROPERTY {
                     }
                 );
                 $self->set_paper($newval);
-                $self->{current_scan_options}
-                  ->add_frontend_option( $name, $newval );
             }
             when ('paper_formats') {
                 $self->{$name} = $newval;
@@ -1090,22 +1090,22 @@ sub pack_widget {
             my $text;
             given ( $opt->{unit} ) {
                 when (SANE_UNIT_PIXEL) {
-                    $text = $d_sane->get('pel')
+                    $text = __('pel')
                 }
                 when (SANE_UNIT_BIT) {
-                    $text = $d_sane->get('bit')
+                    $text = __('bit')
                 }
                 when (SANE_UNIT_MM) {
-                    $text = $d_sane->get('mm')
+                    $text = __('mm')
                 }
                 when (SANE_UNIT_DPI) {
-                    $text = $d_sane->get('dpi')
+                    $text = __('ppi')
                 }
                 when (SANE_UNIT_PERCENT) {
-                    $text = $d_sane->get(q{%})
+                    $text = __(q{%})
                 }
                 when (SANE_UNIT_MICROSECOND) {
-                    $text = $d_sane->get('us')
+                    $text = __('μs')
                 }
             }
             my $label = Gtk3::Label->new($text);
@@ -1524,6 +1524,7 @@ sub set_paper {
     if ( not $paper_profile->num_backend_options ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
+        $self->{current_scan_options}->add_frontend_option( 'paper', $paper );
         $self->signal_emit( 'changed-paper', $paper );
         return;
     }
@@ -1531,11 +1532,16 @@ sub set_paper {
     my $signal;
     $signal = $self->signal_connect(
         'changed-current-scan-options' => sub {
-            $self->signal_handler_disconnect($signal);
-            $self->hide_geometry($options);
-            $self->{paper} = $paper;
-            $self->set( 'profile', undef );
-            $self->signal_emit( 'changed-paper', $paper );
+            my ( $dialog, $profile, $uuid ) = @_;
+            if ( $paper_profile->{uuid} eq $uuid ) {
+                $self->signal_handler_disconnect($signal);
+                $self->hide_geometry($options);
+                $self->{paper} = $paper;
+                $self->{current_scan_options}
+                  ->add_frontend_option( 'paper', $paper );
+                $self->set( 'profile', undef );
+                $self->signal_emit( 'changed-paper', $paper );
+            }
         }
     );
 
@@ -1891,9 +1897,8 @@ sub set_options {
     my $window = Gscan2pdf::Dialog->new(
         'transient-for' => $self,
         title           => $d_sane->get( $opt->{title} ),
-        destroy         => TRUE,
     );
-    my $canvas = Goo::Canvas->new;
+    my $canvas = GooCanvas2::Canvas->new;
     $canvas->set_size_request( $CANVAS_SIZE, $CANVAS_SIZE );
     $canvas->{border} = $CANVAS_BORDER;
     $window->get_content_area->add($canvas);
@@ -1912,8 +1917,15 @@ sub set_options {
             my $fleur = Gtk3::Gdk::Cursor->new('fleur');
             my ( $x, $y ) = to_graph( $widget, $event->x, $event->y );
             $x = int($x) + 1;
-            splice @{ $widget->{val} }, $x, 0, $y;
-            splice @{ $widget->{items} }, $x, 0, add_value( $root, $widget );
+            if ( $x > @{ $widget->{val} } ) {
+                push @{ $widget->{val} }, $y;
+                push @{ $widget->{items} }, add_value( $root, $widget );
+            }
+            else {
+                splice @{ $widget->{val} }, $x, 0, $y;
+                splice @{ $widget->{items} }, $x, 0,
+                  add_value( $root, $widget );
+            }
             update_graph($widget);
             return TRUE;
         }
@@ -1939,7 +1951,6 @@ sub set_options {
             return FALSE;
         }
     );
-    $canvas->can_focus(TRUE);
     $canvas->grab_focus($root);
 
     $canvas->{opt} = $opt;
@@ -1954,25 +1965,18 @@ sub set_options {
           sort { $a <=> $b } @{ $opt->{constraint} };
     }
 
-    # Apply button
-    my $abutton = Gtk3::Button->new_from_stock('gtk-apply');
-    $window->add_button($abutton);
-    $abutton->signal_connect(
-        clicked => sub {
+    $window->add_actions(
+        'gtk-apply' => sub {
             $self->set_option( $opt, $canvas->{val} );
 
  # when INFO_INEXACT is implemented, so that the value is reloaded, check for it
  # here, so that the reloaded value is not overwritten.
             $opt->{val} = $canvas->{val};
             $window->destroy;
-        }
+        },
+        'gtk-cancel' => sub { $window->destroy }
     );
 
-    # Cancel button
-    my $cbutton = Gtk3::Button->new_from_stock('gtk-cancel');
-    $window->add_button($cbutton);
-    $cbutton->signal_connect( clicked => sub { $window->destroy } );
-
 # Have to show the window before updating it otherwise is doesn't know how big it is
     $window->show_all;
     update_graph($canvas);
@@ -1981,8 +1985,12 @@ sub set_options {
 
 sub add_value {
     my ( $root, $canvas ) = @_;
-    my $item = Goo::Canvas::Rect->new(
-        $root, 0, 0, $CANVAS_POINT_SIZE, $CANVAS_POINT_SIZE,
+    my $item = GooCanvas2::CanvasRect->new(
+        parent       => $root,
+        x            => 0,
+        y            => 0,
+        width        => $CANVAS_POINT_SIZE,
+        height       => $CANVAS_POINT_SIZE,
         'fill-color' => 'black',
         'line-width' => 0,
     );
@@ -2122,7 +2130,7 @@ sub update_graph {
       ( $xbounds[1] - $xbounds[0], $ybounds[1] - $ybounds[0] );
 
     # Calculate bounds of canvas
-    my ( $x, $y, $cwidth, $cheight ) = $canvas->allocation->values;
+    my ( $x, $y, $cwidth, $cheight ) = $canvas->get_bounds;
 
     # Calculate scale factors
     my @scale = (
@@ -2162,6 +2170,12 @@ sub set_current_scan_options {
         return;
     }
 
+    # If we have no options set, no need to reset to defaults
+    if ( $self->{current_scan_options}->num_backend_options == 0 ) {
+        $self->add_current_scan_options($profile);
+        return;
+    }
+
     # reload to get defaults before applying profile
     my $signal;
     $self->{current_scan_options} =
@@ -2199,7 +2213,7 @@ sub add_current_scan_options {
     # in case they have to be reloaded.
     # Use the callback to trigger the next loop
     $self->_set_option_profile( $clone, $clone->each_backend_option );
-    return $clone->{uuid};
+    return;
 }
 
 sub _set_option_profile {
@@ -2227,6 +2241,13 @@ sub _set_option_profile {
             }
         }
 
+   # Ignore option if info from previous set_option() reported SANE_INFO_INEXACT
+        if ( defined $self->{option_info}{ $opt->{name} }
+            and $self->{option_info}{ $opt->{name} } & SANE_INFO_INEXACT )
+        {
+            return;
+        }
+
         # Ignore option if value already within tolerance
         if ( Gscan2pdf::Scanner::Options::within_tolerance( $opt, $val ) ) {
             $logger->info(
