@@ -24,10 +24,11 @@ Readonly my $MAXVAL_8_BIT   => 2**$_8_BIT - 1;
 Readonly my $_16_BIT        => 16;
 Readonly my $MAXVAL_16_BIT  => 2**$_16_BIT - 1;
 Readonly my $LARGE_STATUS   => 99;
+Readonly my $NOT_FOUND      => -1;
 my $uuid_object = Data::UUID->new;
 my $EMPTY       = q{};
 
-our $VERSION = '2.5.1';
+our $VERSION = '2.5.4';
 
 my ( $prog_name, $logger, %callback, $_self );
 
@@ -363,8 +364,14 @@ sub check_return_queue {
                 $callback{ $data->{uuid} }{started}->();
             }
             if ( defined $callback{ $data->{uuid} }{finished} ) {
-                $callback{ $data->{uuid} }{finished}
-                  ->( _thaw_deref( $data->{info} ), $data->{status} );
+                if ( $data->{process} eq 'set-option' ) {
+                    $callback{ $data->{uuid} }{finished}
+                      ->( $data->{info}, $data->{status} );
+                }
+                else {
+                    $callback{ $data->{uuid} }{finished}
+                      ->( _thaw_deref( $data->{info} ), $data->{status} );
+                }
                 delete $callback{ $data->{uuid} };
             }
         }
@@ -372,6 +379,39 @@ sub check_return_queue {
     return Glib::SOURCE_CONTINUE;
 }
 
+sub _log2 {
+    my ($n) = @_;
+    return log($n) / log 2;
+}
+
+sub decode_info {
+    my ($info) = @_;
+    if ( $info == 0 ) { return 'none' }
+    my @opts =
+      qw(SANE_INFO_INEXACT SANE_INFO_RELOAD_OPTIONS SANE_INFO_RELOAD_PARAMS);
+    my @this;
+    my $n = _log2($info);
+    if ( $n > int $n ) {
+        $n = int($n) + 1;
+    }
+    my $i = @opts;
+    while ( $n > $i ) {
+        if ( $info >= 2**( $n - 1 ) ) {
+            push @this, q{?};
+            $info -= 2**( $n - 1 );
+        }
+        --$n;
+    }
+    while ( $n > $NOT_FOUND ) {
+        if ( $info >= 2**$n ) {
+            push @this, $opts[$n];
+            $info -= 2**$n;
+        }
+        --$n;
+    }
+    return join ' + ', @this;
+}
+
 sub _thread_main {
     my ($self) = @_;
 
@@ -570,28 +610,29 @@ sub _thread_set_option {
         $status = $_->status;
     };
     if ( $logger->is_info ) {
-        $logger->info( "sane_set_option $index ($opt->{name})"
+        $logger->info(
+                "sane_set_option $index ($opt->{name})"
               . ( $opt->{type} == SANE_TYPE_BUTTON ? $EMPTY : " to $value" )
               . " returned status $status ("
               . Image::Sane::strstatus($status)
               . ') with info '
-              . ( defined $info ? $info : 'undefined' ) );
-    }
-
-    # $info could be undefined if status is invalid
-    if ( defined $info and $info & SANE_INFO_RELOAD_OPTIONS ) {
-        $status = _thread_get_options( $self, $uuid );
-    }
-    else {
-        $self->{return}->enqueue(
-            {
-                type    => 'finished',
-                process => 'set-option',
-                uuid    => $uuid,
-                status  => $status,
-            }
+              . (
+                defined $info
+                ? sprintf( '%d (%s)',
+                    $info, Gscan2pdf::Frontend::Image_Sane::decode_info($info) )
+                : 'undefined'
+              )
         );
     }
+    $self->{return}->enqueue(
+        {
+            type    => 'finished',
+            process => 'set-option',
+            uuid    => $uuid,
+            status  => $status,
+            info    => $info,
+        }
+    );
     return;
 }
 
